/* 
 *      XSitecopy, for managing remote web sites with a GNOME interface.
 *      Copyright (C) 1999, Lee Mallabone <lee0@callnetuk.com
 *                                                                        
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *     
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *     
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  
 */
#include <gnome.h>

/* Gnome message/dialog functions. This contains mostly wrappers for the
 * sitecopy API from the gnome API. 
 */

#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "sites.h"
#include "rcfile.h"
#include "fe_gnome_common.h"
#include "fe_gtk_main.h"
#include "config.h"


#define RCNAME "/." PACKAGE "rc"
#define COPYNAME "/." PACKAGE "/"

extern GtkWidget *app_bar;
extern struct site_t *selected_site;
extern GtkWidget *error_log_list, *error_log_window;

/* For the 'main' upload window */
extern GtkWidget *upload_window, *main_progressbar, *job_progressbar, *error_button;
extern GtkWidget *status_label, *op_label, *file_label, *dir_label, *info_label;
extern GtkWidget *begin_button;

extern struct site_t *all_sites;

/* This value is what makes the entire "job" progress bar possible. */
/* Except it doesn't quite. */
size_t filesize_to_transfer;
float job_progress_so_far = 0.0;

extern GtkWidget *sitecopy;

/*************************************************/

void creation_cb (GnomeDialog *dialog, gint button_number, gpointer data) {
   if (button_number == 0)
     add_site();
   gnome_dialog_close (dialog);
}

/* I really don't like which file i've placed this in... */

int create_site (void) {
   GtkWidget *dialog, *hbox, *top_box, *label;
   extern GtkWidget *sitename_entry;
   extern GtkWidget *localpath_entry, *remotepath_entry;
   
   dialog = gnome_dialog_new (_("Add a new site definition"), GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL, NULL);
   top_box = gtk_vbox_new (FALSE, 3);
   gtk_widget_show (top_box);
   label = gtk_label_new ("This will create a new site definition.\n
			    Give the site a 'friendly' name to refer to it with,
			  and enter the local path that files will be uploaded from.\n
			    The remote path should start with a '/' if it is relative to the 
			    root of the remote site, or a ~/ if it is relative to your login directory.\n
			    NOTE:- Both paths should end with a trailing '/'.\n");
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (top_box), label, FALSE, FALSE, 2);
   
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_widget_show (hbox);
   label = gtk_label_new ("'Friendly' name:");
   gtk_widget_show (label);
   sitename_entry = gtk_entry_new ();
   gtk_widget_show (sitename_entry);
   
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), sitename_entry, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (top_box), hbox, FALSE, FALSE, 2);
   
   /* local dir entry */
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_widget_show (hbox);
   label = gtk_label_new ("Local directory:");
   gtk_widget_show (label);
   localpath_entry = gnome_file_entry_new (NULL, "Enter the local site's root directory.");
   gtk_widget_show (localpath_entry);
   
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), localpath_entry, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (top_box), hbox, FALSE, FALSE, 2);

   /* Remote dir entry */
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_widget_show (hbox);
   label = gtk_label_new ("Remote directory:");
   gtk_widget_show (label);
   remotepath_entry = gtk_entry_new ();
   gtk_widget_show (remotepath_entry);
   
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (hbox), remotepath_entry, FALSE, FALSE, 2);
   gtk_box_pack_start (GTK_BOX (top_box), hbox, FALSE, FALSE, 2);

   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(dialog)->vbox), 
		       top_box, TRUE, TRUE, GNOME_PAD);
   gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
		       GTK_SIGNAL_FUNC (creation_cb), NULL);
   gtk_widget_show (dialog);
   return 1;
}

int fe_gtk_question (char *question, GnomeReplyCallback yes_action) {
   GtkWidget *question_window;
   question_window = gnome_app_question (GNOME_APP (sitecopy), (const gchar *) question,
					 yes_action, NULL);
   gtk_widget_show (question_window);
   return 1;
}
   
/* FIXME: Transfer stuff should perhaps use gnome_dialog natively? */

void fe_transfer_progress (size_t progress, size_t total) {
   
   /*   extern GtkWidget *job_progressbar;*/
   
   float total_percentage;
   float tmp;
   float percentage;
   float div1, div2;

   div1 = (float) progress;
   div2 = (float) total;
   
   percentage = div1 / div2;
/*   g_print ("Percentage passing to progress bar is %f.\n", percentage);*/
 
      gtk_progress_bar_update (GTK_PROGRESS_BAR (main_progressbar), percentage);
      
      tmp = (float) progress / (float) filesize_to_transfer;
      job_progress_so_far = job_progress_so_far + (size_t) tmp;
      total_percentage = (float) job_progress_so_far;
	
      //printf ("\"percentage\" is %f\n", percentage);
      //printf ("\"tmp\" = %f\n", tmp);
      //printf ("\"total_percentage\" is %f\n", total_percentage);
      gtk_progress_bar_update (GTK_PROGRESS_BAR (job_progressbar), tmp);
   while (gtk_events_pending() > 0)
     gtk_main_iteration();
}

void fe_status (char *message) {
   gnome_app_message (GNOME_APP (sitecopy), (const gchar *) message);   
}
   
/* 
 * TODO: I need to add support for yes/no confirmation stuff.
 */

void fe_connecting (void) {
   char *file_status;
   
   file_status = malloc(BUFSIZ);
   sprintf (file_status, "Attempting to Connect to %s...", selected_site->server);
   gtk_label_set (GTK_LABEL (status_label), file_status);
   while (gtk_events_pending() > 0)
     gtk_main_iteration();

}

void fe_connected (void) {
   gtk_label_set (GTK_LABEL (status_label), "Connected.");
   while (gtk_events_pending() > 0)
     gtk_main_iteration();
}

bool fe_can_update (struct site_file_t *file) {
   g_print ("Confirmation given to upload file %s.\n", file->rel_local);
   return true;
}

void fe_updating (struct site_file_t *file) {
   char *file_status;

   file_status = malloc(BUFSIZ);
   sprintf (file_status, "Commiting updates to %s...", selected_site->server);
   gtk_label_set (GTK_LABEL (status_label), file_status);
   if (file->dir) {
      if (file->diff == file_new) {
	 gtk_label_set (GTK_LABEL (op_label), "Creating directory...");
	 gtk_label_set (GTK_LABEL (file_label), file->full_local);
	 /*	    gtk_label_set (GTK_LABEL (dir_label), file->directory); */
	 gtk_label_set (GTK_LABEL (dir_label), "");
      } else {
	 /* can we move dirs yet? */
	 gtk_label_set (GTK_LABEL (op_label), "Deleting directory...");
	 gtk_label_set (GTK_LABEL (dir_label), "");
      }
   } else {
      switch (file->diff) {
       case file_changed:
	 gtk_label_set (GTK_LABEL (op_label), "Uploading...");
	 gtk_label_set (GTK_LABEL (file_label), file->full_local);
	 gtk_label_set (GTK_LABEL (dir_label), file->full_remote);
	 break;
       case file_new:
	 gtk_label_set (GTK_LABEL (op_label), "Uploading...");
	 gtk_label_set (GTK_LABEL (file_label), file->full_local);
	 gtk_label_set (GTK_LABEL (dir_label), file->full_remote);
	 break;
       case file_deleted:
	 gtk_label_set (GTK_LABEL (op_label), "Deleting...");
	 gtk_label_set (GTK_LABEL (file_label), file->full_local);
	 gtk_label_set (GTK_LABEL (dir_label), file->full_remote);
	 break;
       case file_moved:
	 gtk_label_set (GTK_LABEL (op_label), "Moving...");
	 gtk_label_set (GTK_LABEL (file_label), file->old->full_local);
	 gtk_label_set (GTK_LABEL (dir_label), file->directory);
	 break;
       case file_unchanged:
	 gtk_label_set (GTK_LABEL (op_label), "ARG! The file hasn't changed, we shouldn't be doing anything!");
      }
   }
   while (gtk_events_pending() > 0)
     gtk_main_iteration();
}

/* Once a file has been updated, any errors with it are recorded in the 
 * error list, and the progress bar is reset. 
 */
void fe_updated (const struct site_file_t *file, const bool success,
		 const char *error) {
   gchar *finished_status;
   gchar *error_item[2];
   
   finished_status = malloc (BUFSIZ);
   error_item[0] = malloc (BUFSIZ);
   error_item[1] = malloc (BUFSIZ);
   if (!success) {
      strcpy (error_item[0], file->full_local);
      strcpy (error_item[1], error);
      gtk_clist_append (GTK_CLIST (error_log_list), error_item);
/*      printf ("Error \"%s\" with file: %s\n", error, file->full_local);*/
   }
   gtk_progress_bar_update (GTK_PROGRESS_BAR (main_progressbar), 0.0);
   while (gtk_events_pending() > 0)
     gtk_main_iteration();
}

/* TODO: */
void fe_synching (struct site_file_t *file ) {
}
void fe_synched (struct site_file_t *file, bool success) {
}
